/******************************************************************************
 * rtc-test.c: Test Real Time Clock
 *
 * Copyright (C) 2011, MyVoice
 * All Rights reserved.
 *
 * History
 * 2011.06.29	ver 1.00	Preliminary version, first release
 *
 * Basic RTC test
 *  *
 * Software License Agreement
 *
 * The software is owned by MyVoice CAD/CAM Services and/or its suppliers, and is
 * protected under applicable copyright laws.  All rights are reserved.  Any
 * use in violation of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability for the breach
 * of the terms and conditions of this license.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * USE OF THIS SOFTWARE FOR COMMERCIAL DEVELOPMENT AND/OR EDUCATION IS SUBJECT
 * TO A CURRENT END USER LICENSE AGREEMENT (COMMERCIAL OR EDUCATIONAL) WITH
 * MYVOICE CAD/CAM SERVICES.
 *
 */

#include "LPC17xx.h"
#include "tests.h"

#define RTC_OSCF (1<<4)

void rtcTest(void)
{
    int hour, min, sec;
    uint8_t oscfail = 0;
    uint8_t error   = 0;

	printf("RTC Test\n\r");

	/*
	 * Check if the RTC clock has stopped.
	 * If the Oscillator Fail flag is set this means that the clock was halted
	 * due to power loss (or hardware problem).
	 * Reset the flag and check again to see if the clock has started
	 */
	if(LPC_RTC->RTC_AUX & RTC_OSCF)
	{
		oscfail = 1;
		LPC_RTC->RTC_AUX = RTC_OSCF;
		delay(100);
		if(LPC_RTC->RTC_AUX & RTC_OSCF)
		{
			printf("\tRTC Oscillator is not running, not starting\n\r\tError.\n\r");
			errors++;
			return;
		} else
		{
			printf("\tRTC Oscillator was not running, power loss & no battery.\n\r");
		}
	} else {
		hour = LPC_RTC->HOUR;
		min  = LPC_RTC->MIN;
		sec  = LPC_RTC->SEC;

		printf("\tRTC is running, time is: %02d:%02d:%02d\n\r", hour, min, sec);
	}

	printf("\tInitializing time to 9:59:51\n\r");
	LPC_RTC->CCR  =  2; /* Stop the RTC and reset internal oscillator divider */
	LPC_RTC->SEC  = 51;
	LPC_RTC->MIN  = 59;
	LPC_RTC->HOUR = 9;
	LPC_RTC->CCR  =  1; /* and start again */

	printf("\t...wait 5 seconds...\n\r");
	delay(5100); /* wait 5.1 seconds to be sure the second tick has passed */

	hour = LPC_RTC->HOUR;
	min  = LPC_RTC->MIN;
	sec  = LPC_RTC->SEC;

	if(sec != 56)
	{
		printf("\tError: expected time to read 9:59:56 but did read %02d:%02d:%02d\n\r", hour, min, sec);
		error++;
	}
	printf("\t...wait another 5 seconds...\n\r");
	delay(5000);

	hour = LPC_RTC->HOUR;
	min  = LPC_RTC->MIN;
	sec  = LPC_RTC->SEC;

	if((hour != 10) || (min != 0) || (sec != 1))
	{
		printf("\tError: expected time to read 10:00:01 but did read %02d:%02d:%02d\n\r", hour, min, sec);
		error++;
	}
	if(error)
	{
		errors++;
		printf("\tRCT errror: clock is running but RTC does not count\n\r");
	} else if (oscfail)
	{
		printf("\tRTC test passed, no battery detected.\n\r\tPlease insert battery, remove power and check if the RTC is still running\n\r");
	} else
	{
		printf("\tRTC test passed\n");
	}
}
